/**
 * @file cursor.h
 *
 * @author Michal Kopecky
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>)
 * @author Jan Hadrava
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */

#ifndef CURSOR_H_
#define CURSOR_H_

#include "connection.h"
#include "iopccommon/defsPlatform.h"
#include "cursorImpl.h"
#include "iopcmeta/type.h"
#include "iopcmeta/enhancedTypeBase.h"
#include "iopcmeta/enhancedString.h"

#include <string>
#include <map>

namespace iopc {

/**
 * @brief A parameter that cen be passed to a cursor.
 */
class Parameter
{
public:
	enum ParamType {
		IN = 0,
		OUT
	};

	Parameter(ParamType type, const int pos, const Type& varType, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool setNull = false);
	virtual ~Parameter() {}

	void preExec(Cursor* cur);
	void postExec(Cursor* cur);
	void preFetch(Cursor* cur);
	void postFetch(Cursor* cur);

	inline bool isNullValue() { return isNull; }

protected:
	virtual void doPreExec(Cursor* cur) {};
	virtual void doPostExec(Cursor* cur) {};
	virtual void doPreFetch(Cursor* cur) {};
	virtual void doPostFetch(Cursor* cur) {};

	ParamType paramType;
	const int pos;
	const Type& varType;
	const MetadataHolder& attributes;
	bool setNull;
	bool isNull;
};

class SimpleTypeParameter : public Parameter
{
public:
	SimpleTypeParameter(ParamType type, const int pos, const void* varAddr, const Type& varType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);

protected:
	virtual void doPreExec(Cursor* cur);
	virtual void doPreFetch(Cursor* cur);
	virtual void doPostFetch(Cursor* cur);

private:
	const void* varAddr;
	unsigned int maxLen;
	EString tmpStr;
	EWString tmpWStr;

};

class EnhancedTypeParameter : public Parameter
{
public:
	EnhancedTypeParameter(ParamType type, const int pos, EnhancedTypeBase& obj, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
protected:
	virtual void doPreExec(Cursor* cur);
	virtual void doPreFetch(Cursor* cur);
	virtual void doPostFetch(Cursor* cur);

private:
	EnhancedTypeBase& variable;
	unsigned int maxLen;
};

class NullParameter : public Parameter
{
public:
	NullParameter(ParamType type, const int pos, const Type& varType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);

protected:
	virtual void doPreExec(Cursor* cur);

private:
	void* value;
};

/*class NullParameter : public Parameter
{
public:
	NullParameter(const int pos, const Type& varType);
};*/



/**
 * @brief Represents a database cursor for executing SQL statements
 *
 * Also allows user to pass parameters to parametrised queries and fetch
 * results after statement execution.
 *
 * This class is NOT thread safe and is not intended to be used concurrently.
 */
class DLL_EXTERNAL Cursor
{
public:
	enum State {
		CLOSED, OPENED, PREPARED, EXECUTED
	};

private:
	State state;
	CursorImpl* implementation;
	std::string sql;

	typedef std::map<int, Parameter*> ParametersMap;
	typedef ParametersMap::const_iterator ParametersIterator;

	ParametersMap inParams;
	ParametersMap outParams;

	bool fetched;

	void prepareState();
	void checkAndDeleteParam(const int pos, ParametersMap& map);
public:
	void addInParam(const int pos, const void* varAddr, const Type& varType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	void addEnhancedInParam(const int pos, EnhancedTypeBase& obj, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	void addNullInParam(const int pos, const Type& type, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	void addOutParam(const int pos, const void* varAddr, const Type& varType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	void addEnhancedOutParam(const int pos, EnhancedTypeBase& obj, const MetadataHolder& attributes = MetadataHolder::emptyMeta);

	void addParam(Parameter::ParamType direction, const int pos, const void* varAddr, const Type& varType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	void addEnhancedParam(Parameter::ParamType direction, const int pos,  EnhancedTypeBase& obj, const MetadataHolder& attributes = MetadataHolder::emptyMeta);

	Cursor(CursorImpl* cursorImpl, const std::string& sql);
	~Cursor();
	inline bool isOpened() { return state != CLOSED; }
	inline State getState() { return state; }

	inline CursorImpl* getImplementation() { return implementation; }

	void open();
	void prepare();
	void preExecBindNull(const int pos, const Type& varType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);

	void preExecBind(const int pos, const void* varAddr, const unsigned int size, const Type& varType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	void execute();

	void preFetchBind(const int pos, const void* varAddr, const unsigned int size, const Type& varType, bool* isNull = NULL, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	bool fetchNext();
	bool fetchPrev();
	long getPosition();
	void close();

//	virtual void test() = 0;


};

} // namespace iopc

#endif /*CURSOR_H_*/

	/**
	 * Binds the variable to a place in a prepared statement
	 *
	 * @a Prepare() has to be invoked before calling preExecBind
	 * Only these typeclasses can be bound:
	 * <ul>
	 * <li>Type::IOPC_TYPECLASS_SIMPLE</li>
	 * <li>Type::IOPC_TYPECLASS_STRING</li>
	 * <li>Type::IOPC_TYPECLASS_ENHANCED</li>
	 * </ul>
	 *
	 * Supported keys for the optional @a attributes parameter are:
	 * <ul>
	 * <li><b>db.type.length</b> - For the EnhancedString and EnhancedWString types
	 *     specifies the length of the externalized string.</li>
	 * </ul>
	 *
	 * @param pos Position of the parameter in the prepared statement
	 * @param varAddr Address of the variable to be bound
	 * @param vatType Type of the variable
	 * @param attributes Specifies additional data for db mapping
	 *
	 */
	//void preExecBind(const int pos, const void* varAddr, const Type& varType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);

	/**
	 * Binds the enhanced type variable to a place in a prepared statement
	 *
	 * @a Prepare() has to be invoked before calling preExecBind
	 * @param pos Position of the parameter in the prepared statement
	 * @param obj Enhanced variable to be bound
	 * @param maxLen Specifies the length of the externalized string. If the value is not string, maxLen is ignored.
	 */
	//void preExecBind(const int pos, EnhancedTypeBase& obj, unsigned int maxLen = 0);



