/**
 * @file database.h
 *
 * @author Michal Kopecky
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */

#ifndef DATABASE_H_
#define DATABASE_H_

#include <string>
#include "iopccommon/defsPlatform.h"
#include "connection.h"
#include "databaseImpl.h"

namespace iopc {

class Connection;
class DatabaseImpl;
class Driver;


/**
 * @brief Represents a database instance
 *
 * Methods of this class allow user to create and manipulate connections
 * to this database.
 *
 * Classes implementing this interface are thread safe and can be used by more threads concurrently.
 *
 * @ingroup iopcdb
 *
 */
class DLL_EXTERNAL Database {

public:


	/**
	 * @brief Returns a reference to a @a Driver associated with this database
	 */
	virtual Driver& getDriver() =0;

	/**
	 * Implementation should delete the inner instance
	 */
	virtual ~Database() {};

	/**
	 * @brief Commits transactions on all connections
	 */
	virtual void commit() = 0;

	/**
	 * @brief Rolls back transactions on all connections
	 */
	virtual void rollback()= 0;

	/**
	 * @brief Creates a new connection to this database
	 *
	 * Connection must be opened by calling @a Connection::open() before
	 * executing any SQL commands.
	 *
	 * @param connStr A connection string. Its format is defined by the database driver.
	 * Usually it identifies the database user/schema.
	 *
	 * @returns The @a Connection object representing the requested connection.
	 *
	 * @see Connection
	 */
	virtual Connection* getConnection(const std::string& connStr) = 0;

	/**
	 * @brief Releases the connection created with @a Database::getConnection()
	 *
	 * Any connection that was created using @a Database::getConnection()
	 * must be passed to this method when no longer needed.
	 *
	 * @param The connection to be released
	 */
	virtual void returnConnection(Connection* connection) = 0;

	/**
	 * @brief Commits all connections on the database, then disconnects them from database
	 *
	 * The connections are removed from the connection set. (No @a returnConnection() is needed)
	 */
	virtual void disconnect() = 0;

	/**
	 * @brief Returns the driver-side part of the @a Database implemenation.
	 */
	virtual DatabaseImpl* getImplementation() = 0;


	/**
	 * @brief Returns the driver-independent part of the @a Database implementation.
	 */
	virtual Database* getBasicDatabase() = 0;

};


}

#endif /*DATABASE_H_*/
