#ifndef PERSIDENTIFICATION_H_
#define PERSIDENTIFICATION_H_

#include <string>
#include "types/dbType.h"
#include "iopccommon/defs.h"
#include "oidObject.h"
#include "iopcdb/connection.h"
#include "keyValues.h"

#ifndef WIN32
//#include <unordered_map>
#include <ext/hash_map>
#endif /*WIN32*/


namespace iopc {

/**
 * @brief Represents a unique identifier of a persistent object.
 */

class DLL_EXTERNAL PersIdentification {
	friend class DbPtrBase;
public:
	PersIdentification();
	PersIdentification(const Type& type);
	PersIdentification(const PersIdentification& src);
	// TODO: remove loadRealType parameter. It can't be used only with simple views.
	PersIdentification(const Type& type, const OidType& oid, bool loadRealType = true);
	PersIdentification(const Type& type, const KeyValues& keyValues);
	~PersIdentification();
	PersIdentification& operator=(const PersIdentification& src);
	bool operator<(const PersIdentification& id) const;	/* needed by STL containers */
	bool operator==(const PersIdentification& id) const;
	bool operator!=(const PersIdentification& id) const;
	int compare(const PersIdentification& id) const;

public:	/* methods to be used by cache implementations */

	/**
	 * @brief Load object from database.
	 *
	 * @param conn Connection to be used for DB operation.
	 * @param wait Specifies whether the operation should wiat for database locks to be unlocked.
	 * @param lock Specifies whether the persistent copy should be locked in DB exlusively when loaded.
	 * @return Loaded data.
	 */
	DatabaseObject* load(Connection& conn, bool wait, bool lock) const;



	DatabaseObject* load(Cursor& cur) const;

	/**
	 * @brief Delete object from DB.
	 *
	 * @param conn Connection to be used for DB operation.
	 */
	void remove(Connection& conn) const;

	/**
	 * @brief Update object in DB.
	 *
	 * @param conn Connection to be used for DB operation.
	 * @param obj Local copy of object to be updated.
	 */
	void update(Connection& conn, DatabaseObject& obj) const;

	/**
	 * @brief Insert object into DB.
	 *
	 * @param conn Connection to be used for DB operation.
	 * @param obj The object to be inserted into database.
	 */
	void insert(Connection& conn, DatabaseObject& obj) const;

	// virtualise
	//void setSuperType(Connection& conn);

	void initIdentity(Connection& conn, DatabaseObject& obj);

	bool versionMatches(Connection& conn, OidObject& obj) const;



public:
	inline const KeyValues& getKeyValues() const { return keyValues; }

	inline const OidType getOid() const { return oid; }

	inline OidType& getOidRef() { return oid; }

	inline const OidType& getConstOidRef() const { return oid; }

	//inline VirtualOidType getVirtualOid() const { return virtualOid; }

	inline const Type& getType() const { return dbType.getType(); }

	inline const DbType& getDbType() const { return dbType; }

	inline bool isValid() const { return valid && usable; }


	enum IDType {
		OID = 0,
		KEYVALUES
	};

	inline IDType getIDType() const { return idType; }

private:
	/**
	 * Returns real type if oid was specified and loadRealType was set true during initialization.
	 * Otherwise returns the current dbType.
	 */
	DbType& getDesiredDbType(Connection& conn) const;

	void invalidate();

	void setValid();

private:
	//PersIdentification(); // pri presunu do public todo: odkomentovat asserty v dbptrbase

	/** Buffer holding all components of DB of the object. */
	KeyValues keyValues;

	/* Instance specifying type of the object. */
	//const Type* type;

	bool valid;

	/** False, if empty constructor was called, this instance can't be used. The only thing it can do is to be overwritten. */
	bool usable;

	IDType idType;

	OidType oid;

	//VirtualOidType virtualOid;

	/** Wrapper of type with additional method for persisting objects. */
	DbType& dbType;

	/** Load real type that oid represents */
	bool loadRealType;

	/** Cached real dbType, otherwise NULL */
	mutable DbType* realDbType;

public:

	const static PersIdentification nil;
};

} /* iopc */

#ifndef WIN32
namespace __gnu_cxx {

/* this functor is needed by hash_table using PersIdentification as type of its keys. */
template<> struct hash<iopc::PersIdentification>
{
	size_t operator()(const iopc::PersIdentification& persId) const {
		size_t retval = 0;
		//std::string str = persId.getType().getQualifiedName();
		hash<const char*> H;
		hash<iopc::OidTypeInner> HOid;

		if (persId.getIDType() == iopc::PersIdentification::OID) {
			retval += H("oid");
			retval += HOid(persId.getOid());
		} else {
			for (iopc::KeyValues::Iterator it = persId.getKeyValues().begin(); it != persId.getKeyValues().end(); it++) {
				retval += H(it->first.c_str());
				retval += H(it->second->toString().c_str());
			}
		}

		return retval;
	}
};

}/*namespace __gnu_cxx*/
#endif /*WIN32*/

#endif /*PERSIDENTIFICATION_H_*/
