/**
 * @file enhancedTypeBase.h
 *
 * @ingroup iopcmeta
 *
 * @author Petr Cermak
 */

#ifndef ENHANCEDTYPEBASE_H_
#define ENHANCEDTYPEBASE_H_

#include "iopccommon/defs.h"
#include "object.h"

namespace iopc {

/**
 * @brief Base class of all enhanced data types
 *
 * Enhanced data types are classes that mimic behaviour of selected built-in or STL data types.
 * They provide dirty status tracking and the NULL value support.
 *
 * If used as attribute types in reflection-capable classes, they cooperate with the @a Object
 * class to provide dirty status tracking at a object level.
 *
 * @ingroup iopcmeta
 *
 */
class DLL_EXTERNAL EnhancedTypeBase : public Object {
private:
	bool dirty;
	bool* dirtyPtr;
	bool null;
public:
	virtual ~EnhancedTypeBase() {};

	EnhancedTypeBase() : dirty(false), dirtyPtr(NULL),null(true) {}

	/**
	 * @brief Sets a pointer to a boolean value which is updated to true
	 * whenever status of this object becomes 'dirty'
	 *
	 * Used by @a Object to allow the enhanced datatype attributes to set dirty status of the
	 * whole object.
	 */
	void setDirtyPtr(bool* newDirtyPtr);

	// inherited from Object
	virtual void setDirty(bool dirty = true);
	virtual inline bool isDirty() const { return dirty; }
	virtual inline void resetDirty() { setDirty(false); }

	/**
	 * @brief Sets the NULL status of this object
	 *
	 * @param null true - this object is NULL, false - this object is not NULL
	 */
	virtual inline void setNull(bool null = true) {this->null = null; }

	/**
	 * @brief Determines whether this objects represents a NULL value or not
	 */
	virtual inline bool isNull() const { return null; }

	/**
	 * @brief Resets the NULL status of this object to not NULL.
	 */
	virtual inline void resetNull() { setNull(false); }

	/**
	 * @brief Creates a copy of this object's value and returns a pointer to it.
	 *
	 * Type of the copy is the type of the wrapped value (numbers) or char/wchar_t* for EString
	 * or EWString.
	 *
	 * @param maxlength Maximum allocated length of externalised copy. For numbers this value is ignored. For strings this value must by greater than 0.
	 */
	virtual void* externalizeCopy(unsigned int maxlength = 0) = 0;

	/**
	 * @brief Creates an empty buffer for obtaining the value.
	 *
	 * @param length Allocated length of externalised copy. For numbers this value is ignored. For strings this value must by greater than 0.
	 */
	virtual void* externalizeEmptyBuffer(unsigned int length = 0) = 0;

	/**
	 * @brief Updates this object's value from the externalized buffer
	 */
	virtual void updateFromExternalized() = 0;

	/**
	 * @brief Returns externalized buffer size in bytes
	 */
	virtual unsigned int getBufferSize() = 0;

	/**
	 * @brief Returns inner type for specified enhanced type.
	 *
	 * Inner type represents a type that is wrapped by the enhanced data type.
	 * It should be one of the basic C++ types or STL strings.
	 */
	virtual const Type& getInnerType() const = 0;

	/**
	 * @brief Initializes all enhanced data types
	 *
	 * Called by @a TypeManager::init() from @a IopcMeta::init()
	 */
	static void init();


	/**
	 * @brief Returns inner type for specified enhanced type.
	 *
	 * Inner type represents a type that is wrapped by the enhanced data type.
	 * It should be one of the basic C++ types or STL strings.
	 *
	 * Static variant for enhancedType.getInnerType()
	 */
	static const Type& getInnerTypeFor(const Type& enhancedType);
};


}

#endif /*ENHANCEDTYPEBASE_H_*/
